%==========================================================================
% Script file: parabolic_equation_mult_linear 
% This program solves the system of N linear semidiscretized 
% difusion equations C*du/dt+K*u=p(t) using the algorthm of Table 9.3 in 
% the book by John T. Katsikadelis "The Boundary Element Method for
% Engineers and Scientistss", Academic Press,Elsevier, 2016
%%==========================================================================
%%
%  Record of revisions:
%      Date       Programmer         Description of change
%      ====       ==========         =====================
%   15/04/2015    J.T.Katsikadelis    Original code
% ========================================================================
%%
clear all;
%==========================================================================
%% Data
%=========================================================================
%%
N=2;               % number of equations
T=10;              % total time
h=0.1;             % time step 
nmax=ceil(T/h);    % number of time instants
t_tot=h*nmax;
t=h:h:t_tot;
%--------------------------------------------------------------------------
% Initial condition
%--------------------------------------------------------------------------
u0=[1; 0];
%--------------------------------------------------------------------------
% Read Coefficient matrices  C, K
%--------------------------------------------------------------------------
C=[5 4; 4 5];
K=[25 20; 20 20];
%--------------------------------------------------------------------------
% Read external source
%--------------------------------------------------------------------------
p0=[28.5;  24.6];
p=[57/2*exp(-1/10*t).*cos(t)+73/5*exp(-1/10*t).*sin(t)
    123/5*exp(-1/10*t).*cos(t)+31/2*exp(-1/10*t).*sin(t)];
%=================================================================
%% Solution
%=================================================================
q0=inv(C)*(-K*u0+p0);
U0=[q0; u0];
D=[C K
    -h/2*eye(N) eye(N)];
B=[zeros(N)  zeros(N)
    h/2*eye(N) eye(N)];
A=inv(D)*B;
b=inv(D)*[eye(N); zeros(N)];
U=zeros(2*N,nmax);
U(:,1)=A*U0+b*p(:,1);
for i=2:nmax
    U(:,i)=A*U(:,i-1)+b*p(:,i);
end
du=U(1:N,:);     % Derivative of the solution du/dt
u=U(N+1:2*N,:);  % Solution
%=================================================================
%% Plot solution
%=================================================================
t=[0 t];
u=[u0 u];
plot (t,u(:,:),'b-','LineWidth',2);grid; 
 

